@@ -336,6 +336,14 @@ seqParallel <- function(cl=seqGetParallel(), gdsfile, FUN,
                 ans <- FUN(gdsfile, ...)
         }
 
+        if (is.function(.combine))
+        {
+            if (length(formals(args(.combine))) == 1L)
+                .combine(ans)
+            else
+                ans <- .combine(NULL, ans)
+        }
+
     } else if (inherits(cl, "cluster"))
     {
         ## multiple processes with a predefined cluster
@@ -455,7 +463,7 @@ seqParallel <- function(cl=seqGetParallel(), gdsfile, FUN,
                 # open the file
                 if (is.character(gds))
                     gds <- seqOpen(gds, allow.duplicate=TRUE)
-                # save interally
+                # save internally
                 .packageEnv$gfile <- gds
                 .packageEnv$sample.sel <- memDecompress(sel_sample, type="gzip")
                 .packageEnv$variant.sel <- memDecompress(sel_variant, type="gzip")
