@@ -2,7 +2,7 @@
 //
 // SeqArray.cpp: the C++ codes for the SeqArray package
 //
-// Copyright (C) 2013-2022    Xiuwen Zheng
+// Copyright (C) 2013-2024    Xiuwen Zheng
 //
 // This file is part of SeqArray.
 //
@@ -20,17 +20,15 @@
 // If not, see <http://www.gnu.org/licenses/>.
 
 #include "Index.h"
-
-#include <Rinternals.h>
-#include <R_ext/Rdynload.h>
-
 #include <set>
 #include <algorithm>
-
 #include "ReadByVariant.h"
 #include "ReadBySample.h"
 #include <ctype.h>
 
+#include <Rinternals.h>
+#include <R_ext/Rdynload.h>
+
 
 
 // ===========================================================
@@ -316,7 +314,7 @@ COREARRAY_DLL_EXPORT SEXP SEQ_SetSpaceSample2(SEXP gdsfile, SEXP samp_sel,
 						last_I = I;
 					}
 				}
-				if (if_warn && warn_flag) warning(WARN_SEL_INDEX);
+				if (if_warn && warn_flag) warning("%s", WARN_SEL_INDEX);
 				// set values
 				memset((void*)pArray, 0, Count);
 				pI = INTEGER(samp_sel);
@@ -343,7 +341,7 @@ COREARRAY_DLL_EXPORT SEXP SEQ_SetSpaceSample2(SEXP gdsfile, SEXP samp_sel,
 						last_I = I;
 					}
 				}
-				if (if_warn && warn_flag) warning(WARN_SEL_INDEX);
+				if (if_warn && warn_flag) warning("%s", WARN_SEL_INDEX);
 				// get the current index
 				vector<int> Idx;
 				Idx.reserve(Cnt);
@@ -601,7 +599,7 @@ COREARRAY_DLL_EXPORT SEXP SEQ_SetSpaceVariant2(SEXP gdsfile, SEXP var_sel,
 							if_warn = true;
 					}
 				}
-				if (if_warn && warn_flag) warning(WARN_SEL_INDEX);
+				if (if_warn && warn_flag) warning("%s", WARN_SEL_INDEX);
 				// set the structure of selected variants
 				Sel.varTrueNum = num;
 				Sel.varStart = st;
@@ -1482,9 +1480,9 @@ COREARRAY_DLL_EXPORT SEXP SEQ_Debug(SEXP gdsfile)
 		TSelection::TSampStruct *p = Sel.GetStructSample();
 		while (p->length > 0)
 		{
-			Rprintf("    start: %d, length: %d, sel: %p\n", p->offset/ploidy,
-				p->length/ploidy, p->sel);
-			p ++;
+			Rprintf("    start: %d, length: %d, sel: %p\n",
+				int(p->offset/ploidy), int(p->length/ploidy), p->sel);
+			p++;
 		}
 
 		Rprintf("Selected variants:\n");
